@@ -1,10 +1,10 @@
 #ifndef S7_H
 #define S7_H
 
-#define S7_VERSION "11.5"
-#define S7_DATE "8-Sep-2025"
+#define S7_VERSION "11.7"
+#define S7_DATE "19-Nov-2025"
 #define S7_MAJOR_VERSION 11
-#define S7_MINOR_VERSION 5
+#define S7_MINOR_VERSION 7
 
 #include <stdint.h>           /* for int64_t */
 
@@ -543,10 +543,10 @@ s7_pointer s7_signature(s7_scheme *sc, s7_pointer func);                    /* (
 s7_pointer s7_make_signature(s7_scheme *sc, s7_int len, ...);               /* procedure-signature data */
 s7_pointer s7_make_circular_signature(s7_scheme *sc, s7_int cycle_point, s7_int len, ...);
 
-/* possibly unsafe functions: */
+/* unsafe function: */
 s7_pointer s7_make_function(s7_scheme *sc, const char *name, s7_function fnc, s7_int required_args, s7_int optional_args, bool rest_arg, const char *doc);
 
-/* safe functions: */
+/* safe function: */
 s7_pointer s7_make_safe_function(s7_scheme *sc, const char *name, s7_function fnc, s7_int required_args, s7_int optional_args, bool rest_arg, const char *doc);
 s7_pointer s7_make_typed_function(s7_scheme *sc, const char *name, s7_function f,
 				  s7_int required_args, s7_int optional_args, bool rest_arg, const char *doc, s7_pointer signature);
@@ -554,16 +554,16 @@ s7_pointer s7_make_typed_function_with_environment(s7_scheme *sc, const char *na
 						   s7_int required_args, s7_int optional_args, bool rest_arg, const char *doc, 
 						   s7_pointer signature, s7_pointer let);
 
-/* arglist or body possibly unsafe: */
+/* unsafe: */
 s7_pointer s7_define_function(s7_scheme *sc, const char *name, s7_function fnc, s7_int required_args, s7_int optional_args, bool rest_arg, const char *doc);
 
-/* arglist and body safe: */
+/* safe: */
 s7_pointer s7_define_safe_function(s7_scheme *sc, const char *name, s7_function fnc, s7_int required_args, s7_int optional_args, bool rest_arg, const char *doc);
 s7_pointer s7_define_typed_function(s7_scheme *sc, const char *name, s7_function fnc,
 				    s7_int required_args, s7_int optional_args, bool rest_arg,
 				    const char *doc, s7_pointer signature);
 
-/* arglist unsafe or body unsafe: */
+/* unsafe: */
 s7_pointer s7_define_unsafe_typed_function(s7_scheme *sc, const char *name, s7_function fnc,
 					   s7_int required_args, s7_int optional_args, bool rest_arg,
 					   const char *doc, s7_pointer signature);
@@ -578,6 +578,7 @@ s7_pointer s7_make_safe_function_star(s7_scheme *sc, const char *name, s7_functi
 void s7_define_function_star(s7_scheme *sc, const char *name, s7_function fnc, const char *arglist, const char *doc);
 void s7_define_safe_function_star(s7_scheme *sc, const char *name, s7_function fnc, const char *arglist, const char *doc);
 void s7_define_typed_function_star(s7_scheme *sc, const char *name, s7_function fnc, const char *arglist, const char *doc, s7_pointer signature);
+/*  ^ this returns a safe function */
 s7_pointer s7_define_macro(s7_scheme *sc, const char *name, s7_function fnc, s7_int required_args, s7_int optional_args, bool rest_arg, const char *doc);
 s7_pointer s7_define_expansion(s7_scheme *sc, const char *name, s7_function fnc, s7_int required_args, s7_int optional_args, bool rest_arg, const char *doc);
 
@@ -607,7 +608,7 @@ s7_pointer s7_define_expansion(s7_scheme *sc, const char *name, s7_function fnc,
    *   s7_define_macro returns the name as a symbol.
    *
    * Use the "unsafe" definer if the function might call the evaluator itself in some way (s7_apply_function for example),
-   *   or messes with s7's stack.
+   *   or messes with s7's stack (s7_values), or clobbers the argument list.
    */
 
   /* In s7, (define* (name . args) body) or (define name (lambda* args body))
@@ -638,6 +639,15 @@ s7_pointer s7_define_expansion(s7_scheme *sc, const char *name, s7_function fnc,
    *   for C-level functions (as well as optional/rest arguments).
    */
 
+typedef enum {s7_unsafe, s7_semisafe, s7_safe} s7_safety_t;
+s7_pointer s7_defun(s7_scheme *sc, const char *name, /* name=NULL gives anonymous func, return value not symbol */
+		    s7_function f,
+		    s7_int required_args, s7_int optional_args, bool rest_arg,
+		    const char *doc,                 /* NULL for no doc */
+		    s7_pointer signature,            /* NULL for no sig */
+		    s7_pointer let,                  /* NULL for no definition in let */
+		    s7_safety_t type);
+
 s7_pointer s7_apply_function(s7_scheme *sc, s7_pointer fnc, s7_pointer args);
 s7_pointer s7_apply_function_star(s7_scheme *sc, s7_pointer fnc, s7_pointer args);
 
@@ -705,6 +715,7 @@ bool s7_is_c_object(s7_pointer p);
 s7_int s7_c_object_type(s7_pointer obj);
 void *s7_c_object_value(s7_pointer obj);
 void *s7_c_object_value_checked(s7_pointer obj, s7_int type);
+void *s7_c_object_set_value(s7_pointer obj, void *value);
 s7_pointer s7_make_c_object(s7_scheme *sc, s7_int type, void *value);
 s7_pointer s7_make_c_object_with_let(s7_scheme *sc, s7_int type, void *value, s7_pointer let);
 s7_pointer s7_make_c_object_without_gc(s7_scheme *sc, s7_int type, void *value);
@@ -909,7 +920,6 @@ s7_p_ppp_t s7_p_ppp_function(s7_pointer f);
 
 /* -------------------------------------------------------------------------------- */
 
-/* maybe remove these? */
 s7_pointer s7_slot(s7_scheme *sc, s7_pointer symbol);
 s7_pointer s7_slot_value(s7_pointer slot);
 s7_pointer s7_slot_set_value(s7_scheme *sc, s7_pointer slot, s7_pointer value);
@@ -927,7 +937,7 @@ typedef s7_double s7_Double;
 #define s7_object_value       s7_c_object_value
 #define s7_make_object        s7_make_c_object
 #define s7_mark_object        s7_mark
-#define s7_UNSPECIFIED(Sc)    s7_unspecified(Sc)
+#define s7_UNSPECIFIED(Sc)    s7_unspecified(Sc) /* I assume this is still needed by Grace?? deprecated 15 years ago */
 #endif
 
 
@@ -954,6 +964,8 @@ bool s7_is_bignum(s7_pointer obj);
  *
  *        s7 changes
  *
+ * 7-Oct:     s7_safety_t and s7_defun.
+ * 2-Oct:     s7_c_object_set_value.
  * 7-June:    *s7* 'hash-table-missing-key-value and 'iterator-at-end-value.
  *            replace s7_closure* with s7_lambda*.
  * 3-Apr:     c-object-let.
