@@ -47,11 +47,11 @@
 
 #include "snd-strings.h"
 
-#define SND_DATE "8-Sep-25"
+#define SND_DATE "19-Nov-25"
 #ifndef SND_VERSION
-#define SND_VERSION "25.7"
+#define SND_VERSION "25.9"
 #endif
 #define SND_MAJOR_VERSION "25"
-#define SND_MINOR_VERSION "7"
+#define SND_MINOR_VERSION "9"
 
 #endif
